@Echo Off
REM
REM iInstall.cmd, version 3.0.
REM --------------------------
REM Jesper Wachs, February 2016.
REM
REM CMD file will install a savefile containing software on an iSeries.
REM For more information, see article here http://www.wachs.dk/iinstall-cmd-an-iseries-tool
REM
REM Version 3.0, February, 2016:
REM ----------------------------
REM - Colouring engine changed to a more robust one.
REM - Option to have a 'License Agreement' text.
REM
REM Version 2.0, November, 2014:
REM ----------------------------
REM - Full error log file (iInstall.log) will be created upon errors.
REM - Parameters for the script is now in a seperate CMD file called iInstallParm.cmd
REM   in order to make it easier to build the installation script.
REM - Texts are now displayed in color.
REM - Spelling errors corrected.
REM - Wrongly used env var 'savlib'. Replaced by 'rstlib'.
REM - Checks user id and password during system check.
REM 
REM Version 1.1. 5th of July, 2014:
REM -------------------------------
REM - First version as a seperate tool and not just for my own tools.
REM - Able to install a ToolBox file.
REM
REM Version 1.0:
REM ------------
REM - Initial version. Made entirely for installing my own tools.
REM
setlocal DisableDelayedExpansion
Cls
Call :SetUpConfig
REM
REM Parameters for the installation are in a seperate cmd file, that
REM will setup the needed environment variables.
REM
If not exist "%~dp0%~n0Parm.cmd" (
Call :ErrHdr
Call :ErrDetLin "The file:"
Call :ErrDetLin ""
Call :ErrDetLin "%~dp0%~n0Parm.cmd"
Call :ErrDetLin ""
Call :ErrDetLin "Could not be found. Contact your supplier."
Call :ErrFtr
Pause
Goto Exit
)
CALL "%~dp0%~n0Parm.cmd" "StArTeD"
REM
REM Initial error check.
REM
if "%utilityname%" == "" Goto SetUpErr
if "%savf%" == "" Goto SetUpErr
if "%savfext%" == "" Goto SetUpErr
if "%lib%" == "" Goto SetUpErr
if "%rstlib%" == "" Goto SetUpErr
if "%libtext%" == "" Goto SetUpErr
Goto AllGood

:SetUpErr
REM
REM Error with missing parameters.
REM
Call :ErrHdr
Call :ErrDetLin "Information on what and how to install it, is missing in the script."
Call :ErrDetLin ""
Call :ErrDetLin "Contact your supplier."
Call :ErrFtr
Pause
Goto Exit

:AllGood
set q=^"
if not exist "%~dp0%savf%%savfext%" (
Call :ErrHdr
Call :ErrDetLin "The pc savefile:"
Call :ErrDetLin ""
Call :ErrDetLin "%~dp0%savf%%savfext%"
Call :ErrDetLin ""
Call :ErrDetLin "could not be found! Contact your supplier."
Call :ErrFtr
Pause
Goto Exit
)

CLS
Echo.
call :c 02 "This "
call :c 0A "iInstall.cmd"
call :c 02 " script will install the software:" /n
Echo.
call :c 0F "%utilityname%" /n
Echo.
call :c 02 "on an iSeries of your choice." /n
Echo.
call :c 02 "This script will ask you for the following information:" /n
Echo.
call :c 02 " - Name or IP of iSeries to install on." /n
call :c 02 " - A iSeries login and password that must have authority to use the" /n
call :c 02 "   following command:" /n
Echo.
call :c 02 " - Create Library (CRTLIB)" /n
call :c 02 " - Restore Object (RSTOBJ)" /n
call :c 02 " - Copy File (CPYF)" /n
Echo.
call :c 02 "Press CTRL-C or CTRL-BREAK to stop, else press any key to continue." /n
Echo.
Pause

REM
REM Show License Agreement if one exits, and have user confirm it.
REM
if "%lictext1%" == "" Goto StartInst
CLS
Echo.
call :c 02 "This is the License Agreement for the software "
call :c 0F "%utilityname%:" /n
Echo.
REM
REM Loop through all the variables and print them.
REM
setlocal EnableDelayedExpansion
set /a "index=1"
:NextLine
set thisline=lictext%index%
if not "!%thisline%!" == "" (
	if "!%thisline%!" == "*" Echo.
	if not "!%thisline%!" == "*" call :c 02 "!%thisline%!" /n
	set /a "index+=1"
	Goto NextLine
)
Echo.
call :c 02 "To accept these terms, type Y and press ENTER." /n
call :c 02 "To decline these terms, type N and press ENTER." /n
:AskAgain
set /P accept= Type Y or N:

if /i "%accept%" == "y" Goto StartInst
if /i "%accept%" == "n" (
Echo.
call :c 02 "The installation has been terminated." /n
Echo.
Pause
Goto End
)
Goto AskAgain

REM
REM Start installation.
REM
:StartInst
setlocal DisableDelayedExpansion
Echo.
call :c 02 "OK - Here we go." /n
Echo.
set /p iseriesname= Enter DNS-name or IP of iSeries:
Echo.
set /p user= Enter id for iSeries userprofile:
Echo.
set /p pwd= Enter password for iSeries userprofile:
Echo.
Cls
Echo.
call :c 02 "Installation script will install on .: %iseriesname%" /n
call :c 02 "using profile and password ..........: %user% / *****" /n
Echo.
call :c 02 "Press any key to continue or press CTRL-C or CTRL-BREAK to stop" /n
call :c 02 "installation." /n
Echo.
Pause
REM
REM Connect and test if target is an iSeries
REM
Echo.
call :c 02 "Verifying user id, password and system." /n
Echo.
call :c 02 "Please wait..." /n
Echo.
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote system >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Execute FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Check if user was rejected.
REM
FIND "530 " < "%logfile2%" > nul
If not errorlevel == 1 (
Echo.
Call :ErrHdr
Call :ErrDetLin "User id/password is not correct on target system."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
REM
REM If user was not rejected, user must be logged on.
REM
FIND "230 " < "%logfile2%" > nul
If errorlevel == 1 (
Echo.
Call :ErrHdr
Call :ErrDetLin "User could not be logged on target system."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
REM
REM Check that we are connected to an iSeries.
REM
FIND "OS/400" < "%logfile2%" > nul
If errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "Target system is not an iSeries."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
call :c 02 "Verification successful." /n
Echo.

REM
REM Figure out if lib exists.
REM
if "%rstlib%" == "" set rstlib = %lib%
Echo.
call :c 02 "Verifying if library %rstlib% exists." /n
Echo.
call :c 02 "Please wait..." /n
Echo.
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd chkobj %rstlib% *lib >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Execute FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Check if library exists.
REM
FIND "550-" < "%logfile2%" > nul
If errorlevel == 1 Goto C250
Goto Cont1

:C250
REM
REM In this case, the system must have returned 250 which means ok.
REM
FIND "250" < "%logfile2%" > nul
if errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "Test for lib %rstlib% did not execute correctly."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
Echo.
call :c 02 "The library %rstlib% exists on %iseriesname% and will be used to restore" /n
call :c 02 "objects into." /n
Echo.
call :c 02 "Press any key to continue or press CTRL-C or CTRL-BREAK to stop" /n
call :c 02 "installation." /n
Echo.
Pause
Goto XFer

:Cont1
REM
REM Lib does not exist, create it.
REM
Echo.
call :c 02 "The library %rstlib% does not exist. It will be created." /n
Echo.
call :c 02 "Press any key to continue or press CTRL-C or CTRL-BREAK to stop" /n
call :c 02 "installation." /n
Echo.
Pause
Echo.
call :c 02 "Creating library %rstlib%." /n
Echo.
call :c 02 "Please wait..." /n
Echo.
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd crtlib %rstlib% TYPE(*TEST) TEXT('%libtext%') >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Execute FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Check that library was created.
REM
FIND "250" < "%logfile2%" > nul
If errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "Could not create the library %rstlib%."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
call :c 02 "Library %rstlib% created successful." /n
Echo.

:XFer
REM
REM Transfer the savfile to iSeries.
REM
Echo.
call :c 02 "Transfering savefile to system and restoring objects." /n
Echo.
call :c 02 "Please wait..." /n
Echo.
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd crtsavf qtemp/%savf% >> "%scriptfile%"
Echo bin >> "%scriptfile%"
Echo quote site namefmt 1 >> "%scriptfile%"
Echo put "%~dp0%savf%%savfext%" /QSYS.LIB/QTEMP.LIB/%savf%.SAVF >> "%scriptfile%"
Echo quote rcmd RSTOBJ *ALL SAVLIB(%lib%) DEV(*SAVF) SAVF(QTEMP/%savf%) RSTLIB(%rstlib%) >> "%scriptfile%"
Echo quote rcmd dltf qtemp/%savf% >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Execute FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Check that we are connected to an iSeries.
REM
FIND "550" < "%logfile2%" > nul
If errorlevel == 1 Goto Cont2
Echo.
Call :ErrHdr
Call :ErrDetLin "Could not transfer savefile %savf% or restore objects into %lib%."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:Cont2
call :c 02 "Transfer and restore successful." /n
Echo.

REM
REM Verify if ToolBox is installed.
REM
REM If ToolBox is blank, then skip.
REM
if "%tbfile%" == "" Goto Cont4
Echo.
call :c 02 "Checking if you have ToolBox installed." /n
Echo.
call :c 02 "Please wait..." /n
Echo.
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd chkobj %rstlib%/TBPF *FiLE >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Execute FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
FIND "250" < "%logfile2%" > nul
If errorlevel == 1 (
call :c 02 "You do not have ToolBox installed. You can download it from the website." /n
Echo.
call :c 02 "A file called %tbfile% exists in library %rstlib%. When you have" /n
call :c 02 "installed ToolBox, you can copy this file to the file TBPF in order" /n
call :c 02 "to have this tool show up in ToolBox." /n
Echo.
Pause
Goto Cont4
)

:Cont3
REM
REM Copy content of ToolBox file, if target exists.
REM
Echo.
call :c 02 "Installing into ToolBox." /n
Echo.
call :c 02 "Please wait..." /n
Echo.
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd cpyf %rstlib%/%tbfile% TOFILE(%rstlib%/TBPF) MBROPT(*ADD) ERRLVL(*NOMAX) >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Execute FTP
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Check that copy was successfull.
REM
FIND "550" < "%logfile2%" > nul
If errorlevel == 1 Goto Cont4
Call :ErrHdr
Call :ErrDetLin "Could not copy ToolBox file content."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:Cont4
Echo.
call :c 02 "Installation into ToolBox successful." /n
Echo.

REM
REM Ok
REM
CLS
Echo.
call :c 02 "The software:" /n
Echo.
call :c 0F "%utilityname%" /n
Echo.
call :c 02 "was installed with success." /n
Echo.
call :c 02 "Installation was performed by "
call :c 0A "iInstall.cmd"
call :c 02 " from Jesper Wachs." /n
call :c 02 "For further information, see http://www.wachs.dk/?p=175" /n
Echo.
Pause
Goto End
:Exit
REM
REM Error Exit
REM
Echo.
call :c 02 "The installation has been terminated." /n
Echo.
call :c 02 "A logfile has been created, and you will find it here:" /n
Echo.
Echo %logfile%
Echo.
call :c 02 "Contact your supplier to get help." /n
Echo.
Call :ResetEnvVars
Pause
Exit /b


REM
REM Cleanup and terminate.
REM
:End
if exist "%logfile%"  Del "%logfile%"
Call :ResetEnvVars
exit /b
REM
REM Sub functions
REM
REM Reset Environment variables.
REM ----------------------------
:ResetEnvVars
if exist "%scriptfile%"  Del "%scriptfile%"
if exist "%logfile2%" Del "%logfile2%"
set utilityname=
set savf=
set savfext=
set lib=
set rstlib=
set libtext=
set tbfile=
set thisline=
setlocal EnableDelayedExpansion
set /a "index=1"
:ResetLbl1
set thisline=lictext%index%
if not "!%thisline%!" == "" (
	echo her!
	set %thisline%=
	set /a "index+=1"
	Goto ResetLbl1
)
set scriptfile=
set logfile=
set logfile2=
set iseriesname=
set user=
set pwd=
set q=

Exit /b

REM
REM Internal config of this script.
REM -------------------------------
:SetUpConfig
Set scriptfile=%~dp0%~n0.script
Set logfile=%~dp0%~n0.log
Set logfile2=%~dp0%~n02.log
Exit /b

REM
REM Write Error Header
REM on screen and in file.
REM ----------------------
:ErrHdr
REM
REM For the error log file, the configuration and other info is written.
REM
Echo iInstall error log. > "%logfile%"
Echo ------------------- >> "%logfile%"
Echo Date/time: %date% %time% >> "%logfile%"
Echo ======================================================================== >> "%logfile%"
Echo Configuration: >> "%logfile%"
Echo -------------- >> "%logfile%"
Echo Utility name......: %utilityname% >> "%logfile%"
Echo Savefile name.....: %savf% >> "%logfile%"
Echo Savefile extension: %savfext% >> "%logfile%"
Echo Save library......: %lib% >> "%logfile%"
Echo Restore library...: %rstlib% >> "%logfile%"
Echo Library text......: %libtext% >> "%logfile%"
Echo ToolBox file......: %tbfile% >> "%logfile%"
Echo iInstall.cmd load.: %~dp0%~n0%~x0 >> "%logfile%"
Echo FTP script file...: %scriptfile% >> "%logfile%"
Echo iSeries name......: %iseriesname% >> "%logfile%"
Echo User id...........: %user% >> "%logfile%"

Call :ErrDetLin "========================================================================" 04
Call :ErrDetLin "Error occured." 0C
Call :ErrDetLin "--------------" 0C
Exit /b

REM
REM Write Error detail line
REM on screen and in file.
REM -----------------------
:ErrDetLin Str
if "%~1" == "" Echo.
if not "%~1" == "" (
  if "%2" == "" call :c 02 "%~1" /n
  if not "%2" == "" call :c %2 "%~1" /n
)
if "%~1" == "" Echo. >> "%logfile%"
if not "%~1" == "" Echo %~1 >> "%logfile%"
Exit /b

REM
REM Write Error Footer
REM on screen and in file.
REM ----------------------
:ErrFtr
Call :ErrDetLin "========================================================================" 04
Exit /b

REM
REM Write FTP error log
REM on screen and in file.
REM ----------------------
:ErrFTPLog
Call :ErrDetLin ""
Call :ErrDetLin "Check the log below:"
Call :ErrDetLin "--------------------------[ Start of log ]------------------------------" 04
Type "%logfile2%"
Type "%logfile2%" >> "%logfile%"
Call :ErrDetLin "---------------------------[ End of log ]-------------------------------" 04
Exit /b

REM
REM Coloring of text.
REM -----------------
:C
SetLocal EnableExtensions EnableDelayedExpansion
pushd
Subst `: "!Temp!" >Nul & `: & Cd \
SetLocal DisableDelayedExpansion
Echo(|(Pause >Nul &Findstr "^" >`)
Cmd /A /D /C Set /P "=." >>` <Nul
For /F %%# In (
'"Prompt $H &For %%_ In (_) Do Rem"') Do (
Cmd /A /D /C Set /P "=%%# %%#" <Nul >`.1
Copy /Y `.1 /B + `.1 /B + `.1 /B `.3 /B >Nul
Copy /Y `.1 /B + `.1 /B + `.3 /B `.5 /B >Nul
Copy /Y `.1 /B + `.1 /B + `.5 /B `.7 /B >Nul
)
:__Color
Set "Text=%~2"
If Not Defined Text (Set Text=^")
SetLocal EnableDelayedExpansion
For %%_ In ("&" "|" ">" "<"
) Do Set "Text=!Text:%%~_=^%%~_!"
Set /P "LF=" <` &Set "LF=!LF:~0,1!"
For %%# in ("!LF!") Do For %%_ In (
\ / :) Do Set "Text=!Text:%%_=%%~#%%_%%~#!"
For /F delims^=^ eol^= %%# in ("!Text!") Do (
If #==#! EndLocal
If \==%%# (Findstr /A:%~1 . \` Nul
Type `.3) Else If /==%%# (Findstr /A:%~1 . /.\` Nul
Type `.5) Else (Cmd /A /D /C Echo %%#\..\`>`.dat
Findstr /F:`.dat /A:%~1 .
Type `.7))
If "/n"=="%~3" (Shift
Echo()
Shift
Shift
If ""=="%~1" Del ` `.1 `.3 `.5 `.7 `.dat & popd & Subst `: /d &Goto :Eof
Goto :__Color
